# Copyright © 2020 Matthew Burkard
#
# This file is part of Chroma Formatter.
#
# Chroma Formatter is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Chroma Formatter is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Chroma Formatter.  If not, see <https://www.gnu.org/licenses/>.
import logging
import re
import sys
import unittest
from pathlib import Path

from chromaformatter import ChromaFormatter, Colors


class UtilTest(unittest.TestCase):

    def __init__(self, *args) -> None:
        self.log_path = 'log/test.log'
        self.log = logging.getLogger()
        log_format = (
            f'{Colors.Fore.GREEN}[%(asctime)-0s]'
            f'{Colors.LEVEL_COLOR}[%(levelname)-0s]'
            f'{Colors.Fore.MAGENTA}[%(filename)-0s:'
            f'%(lineno)-0d]'
            f'{Colors.LEVEL_COLOR}: %(message)s'
        )
        self.formatter = ChromaFormatter(
            log_format,
            Colors.Fore.WHITE,
            Colors.LEVEL_COLOR
        )
        file_handler = logging.FileHandler(self.log_path, mode='w')
        file_handler.setFormatter(self.formatter)
        stream_handler = logging.StreamHandler(stream=sys.stdout)
        stream_handler.setFormatter(self.formatter)
        self.log.addHandler(stream_handler)
        self.log.addHandler(file_handler)
        self.log.setLevel(logging.DEBUG)
        super(UtilTest, self).__init__(*args)

    def test_formatter_style(self) -> None:
        # noinspection PyProtectedMember
        self.assertEqual(
            self.formatter._style._fmt.encode(),
            f'{Colors.Fore.GREEN}[%(asctime)-0s]$LEVEL[%(levelname)-0s]'
            f'{Colors.Fore.MAGENTA}[%(filename)-0s:%(lineno)-0d]'
            f'{Colors.LEVEL_COLOR}: %(message)s'
            f'{Colors.Style.RESET_ALL}'.encode()
        )

    def test_log_color(self) -> None:
        # Test that a log message is colored as it should be.
        self.log.info(
            'It was not until [%s] that light was identified as the source of'
            ' the color sensation.',
            'Newton'
        )
        file_name = Path(__file__).name
        with open(self.log_path) as f:
            log_file_text = f.read()
            # Replace timestamp time with the text 'timestamp' to check
            # against.
            log_file_text = re.sub(
                r'\[\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2},\d{3}]',
                '[timestamp]',
                log_file_text
            )
            # Replace the line number with the text 'lineno' to check
            # against.
            log_file_text = re.sub(
                fr'\[({file_name}):\d+]',
                r'[\1:lineno]',
                log_file_text
            )
            self.assertEqual(
                log_file_text.encode(),
                f'{Colors.Fore.GREEN}[timestamp]{Colors.Fore.CYAN}[INFO]'
                f'{Colors.Fore.MAGENTA}[{file_name}:lineno]'
                f'{Colors.Fore.CYAN}: It was not until'
                f' [{Colors.Fore.WHITE}Newton{Colors.Fore.CYAN}]'
                f' that light was identified as the source'
                f' of the color sensation.{Colors.Style.RESET_ALL}\n'.encode()
            )

    def test_something(self) -> None:
        # Test that formatter is updated based on log.
        # noinspection PyProtectedMember
        self.assertEqual(
            self.formatter._style._fmt.encode(),
            f'{Colors.Fore.GREEN}[%(asctime)-0s]'
            f'{Colors.Fore.CYAN}[%(levelname)-0s]'
            f'{Colors.Fore.MAGENTA}[%(filename)-0s:%(lineno)-0d]'
            f'{Colors.Fore.CYAN}: %(message)s'
            f'{Colors.Style.RESET_ALL}'.encode()
        )


if __name__ == '__main__':
    unittest.main()



# Copyright © 2020 Matthew Burkard
#
# This file is part of Chroma Formatter.
#
# Chroma Formatter is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Chroma Formatter is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Chroma Formatter.  If not, see <https://www.gnu.org/licenses/>.

from setuptools import setup

setup(
    name='chromaformatter',
    version='5.0.0',
    url='https://gitlab.com/mburkard/chroma-formatter',
    license='GNU General Public License v3 (GPLv3)',
    author='Matthew Burkard',
    author_email='matthewjburkard@gmail.com',
    description='Wrapper for the Python logging module to add color.',
    package_dir={'': 'src'},
    packages=['chromaformatter'],
    long_description_content_type='text/markdown',
    long_description=open('README.md').read(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)'
    ],
    install_requires=['colorama'],
    zip_safe=False
)



# Copyright © 2020 Matthew Burkard
#
# This file is part of Chroma Formatter.
#
# Chroma Formatter is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Chroma Formatter is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Chroma Formatter.  If not, see <https://www.gnu.org/licenses/>.

import re
from logging import Formatter, DEBUG, INFO, WARNING, ERROR, CRITICAL, LogRecord
from typing import Dict, Optional

import colorama

__all__ = (
    'ChromaFormatter',
    'Colors',
)


class Colors:
    Fore = colorama.Fore
    Back = colorama.Back
    Style = colorama.Style
    LEVEL_COLOR = '$LEVEL'


# noinspection PyProtectedMember
class ChromaFormatter(Formatter):
    """Extended logging.Formatter to add colors and styles."""

    def __init__(
            self,
            fmt: str,
            arg_start_color: Optional[str] = None,
            arg_end_color: Optional[str] = None
    ) -> None:
        """Set ChromaFormatter properties.

        :param fmt: The format string to determine how logs will appear.
        :param arg_start_color: Color of formatted arguments.
        :param arg_end_color: Color after formatted arguments.
        """
        fmt = f'{fmt}{Colors.Style.RESET_ALL}'
        super().__init__(fmt)
        self._original_style_fmt = self._style._fmt
        self.arg_start_color = arg_start_color
        self.arg_end_color = arg_end_color

        self.color_map: Dict[int, str] = {
            DEBUG: Colors.Fore.BLUE,
            INFO: Colors.Fore.CYAN,
            WARNING: Colors.Fore.YELLOW,
            # ERROR: Colors.Fore.LIGHTRED_EX,
            # CRITICAL: Colors.Fore.RED
            CRITICAL: Colors.Fore.LIGHTRED_EX,
            ERROR: Colors.Fore.RED
        }

    def format(self, record: LogRecord) -> str:
        """Format and add colors to a log record.

        :param record: LogRecord to format and color.
        :return: The complete log record formatted and colored.
        """
        _init_record(record)
        if record.levelno not in self.color_map:
            return super(ChromaFormatter, self).format(record)
        self._style._fmt = self._original_style_fmt
        level_color = self.color_map[record.levelno]
        # Color the record msg.
        if record.args and self.arg_start_color and self.arg_end_color:
            record.msg = re.sub(
                r'(?<!%)%([-0.\d]*)([sd])',
                fr'{self.arg_start_color}%\1\2{self.arg_end_color}',
                record.msg
            ).replace('$LEVEL', level_color)
        self._style._fmt = self._style._fmt.replace(
            Colors.LEVEL_COLOR,
            level_color
        )
        return super(ChromaFormatter, self).format(record)


def _init_record(record: LogRecord) -> None:
    """Make certain a LogRecord will be using its original values.

    First time called will add original values to a LogRecord as new
    parameters. Subsequent calls will reset a LogRecords parameters back
    to the originals.

    This is so that other handlers won't have the colors inserted from
    previous ones.

    :param record: LogRecord to load with initial values.
    """
    record.msg = str(record.msg)
    try:
        if record.consumed:
            record.msg = record.original_msg
            record.args = record.original_args
    except AttributeError:
        record.original_msg = record.msg
        record.original_args = record.args
        record.consumed = True
